package gregapi.block;

import static gregapi.data.CS.*;
import gregapi.data.LH;
import gregapi.data.OP;
import gregapi.util.ST;
import gregapi.util.UT;

import java.util.List;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.block.BlockStaticLiquid;
import net.minecraft.block.material.Material;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.StatCollector;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.util.ForgeDirection;
import cpw.mods.fml.common.registry.GameRegistry;

/**
 * @author Gregorius Techneticies
 */
public abstract class BlockBase extends Block {
	public final String mNameInternal;
	
	public BlockBase(Class<? extends ItemBlock> aItemClass, String aNameInternal, Material aMaterial, SoundType aSoundType) {
		super(aMaterial);
		setStepSound(aSoundType);
		setBlockName(mNameInternal = aNameInternal);
		GameRegistry.registerBlock(this, aItemClass, getUnlocalizedName());
		LH.add(mNameInternal+"."+W+".name", "Any Sub-Block of this one");
	}
	
	@Override public final String getUnlocalizedName() {return mNameInternal;}
	@Override public String getLocalizedName() {return StatCollector.translateToLocal(mNameInternal+ ".name");}
	@Override public String getHarvestTool(int aMeta) {return TOOL_pickaxe;}
	@Override public int getHarvestLevel(int aMeta) {return 0;}
	@Override public boolean canSilkHarvest() {return T;}
	@Override public boolean isToolEffective(String aType, int aMeta) {return getHarvestTool(aMeta).equals(aType);}
	@Override public boolean canBeReplacedByLeaves(IBlockAccess aWorld, int aX, int aY, int aZ) {return F;}
	@Override public boolean isNormalCube(IBlockAccess aWorld, int aX, int aY, int aZ)  {return T;}
	@Override public boolean renderAsNormalBlock() {return T;}
	@Override public boolean isOpaqueCube() {return T;}
	@Override public boolean isSideSolid(IBlockAccess aWorld, int aX, int aY, int aZ, ForgeDirection aDirection) {return T;}
	@Override public int damageDropped(int aMeta) {return aMeta;}
	@Override public int quantityDropped(Random par1Random) {return 1;}
	@Override public ItemStack createStackedBlock(int aMeta) {return ST.make(this, 1, damageDropped(aMeta));}
	@Override public int getDamageValue(World aWorld, int aX, int aY, int aZ) {return aWorld.getBlockMetadata(aX, aY, aZ);}
	@Override public int getLightOpacity() {return LIGHT_OPACITY_MAX;}
	@Override public Item getItemDropped(int aMeta, Random aRandom, int aFortune) {return Item.getItemFromBlock(this);}
	@Override public Item getItem(World aWorld, int aX, int aY, int aZ) {return Item.getItemFromBlock(this);}
	@Override public void registerBlockIcons(IIconRegister aIconRegister) {/**/}
	@Override public boolean canCreatureSpawn(EnumCreatureType type, IBlockAccess aWorld, int aX, int aY, int aZ) {return canCreatureSpawn(aWorld.getBlockMetadata(aX, aY, aZ));}
	@Override public boolean isFireSource(World aWorld, int aX, int aY, int aZ, ForgeDirection aSide) {return F;}
	@Override public boolean isFlammable(IBlockAccess aWorld, int aX, int aY, int aZ, ForgeDirection aSide) {return F;}
	@Override public int getFlammability(IBlockAccess aWorld, int aX, int aY, int aZ, ForgeDirection aSide) {return 0;}
	@Override public int getFireSpreadSpeed(IBlockAccess aWorld, int aX, int aY, int aZ, ForgeDirection aSide) {return 0;}
	@Override public Material getMaterial() {return blockMaterial.getCanBurn() && !isFlammable(null, 0, 0, 0, FORGE_DIR[SIDE_ANY]) && (new Throwable().getStackTrace()[1].getClassName()) == BlockStaticLiquid.class.getName() ? Material.iron : blockMaterial;}
	@Override public float getExplosionResistance(Entity aEntity, World aWorld, int aX, int aY, int aZ, double eX, double eY, double eZ) {return getExplosionResistance(aWorld.getBlockMetadata(aX, aY, aZ));}
	@Override public float getExplosionResistance(Entity aEntity) {return getExplosionResistance(0);}
	
	public boolean doesWalkSpeed(short aMeta) {return this instanceof IBlockOnWalkOver;}
	public void addInformation(ItemStack aStack, int aMeta, EntityPlayer aPlayer, List aList, boolean aF3_H) {/**/}
	public float getExplosionResistance(int aMeta) {return 10.0F;}
	public boolean canCreatureSpawn(int aMeta) {return F;}
	public int getItemStackLimit(ItemStack aStack) {return UT.Code.bindStack(OP.block.mDefaultStackSize);}
	public ItemStack onItemRightClick(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {return aStack;}
}