package gregapi.block.misc;

import static gregapi.data.CS.*;
import static net.minecraftforge.common.EnumPlantType.Water;
import gregapi.block.BlockBaseMeta;
import gregapi.block.ItemBlockBase;
import gregapi.render.*;
import gregapi.util.ST;
import gregapi.util.UT;

import java.util.List;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.item.EntityBoat;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.IIcon;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.EnumPlantType;
import net.minecraftforge.common.IPlantable;
import net.minecraftforge.common.util.ForgeDirection;

public class BlockBaseLilyPad extends BlockBaseMeta implements IPlantable, IRenderedBlock {
	public BlockBaseLilyPad(Class<? extends ItemBlock> aItemClass, String aNameInternal, Material aMaterial, SoundType aSoundType, IIconContainer[] aIcons) {
		super(ItemBlockBase.class, aNameInternal, Material.plants, soundTypeGrass, aIcons);
		setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 0.015625F, 1.0F);
	}
	
	@Override public String getHarvestTool(int aMeta) {return TOOL_sword;}
	@Override public int getHarvestLevel(int aMeta) {return 0;}
	@Override public void addCollisionBoxesToList(World aWorld, int aX, int aY, int aZ, AxisAlignedBB aAABB, List aList, Entity aEntity) {if (!(aEntity instanceof EntityBoat)) super.addCollisionBoxesToList(aWorld, aX, aY, aZ, aAABB, aList, aEntity);}
	@Override public boolean canBlockStay(World aWorld, int aX, int aY, int aZ) {return aY >= 0 && aY < 256 && aWorld.getBlock(aX, aY - 1, aZ).getMaterial() == Material.water && aWorld.getBlockMetadata(aX, aY - 1, aZ) == 0;}
	@Override public boolean canPlaceBlockAt(World aWorld, int aX, int aY, int aZ) {return super.canPlaceBlockAt(aWorld, aX, aY, aZ) && canBlockStay(aWorld, aX, aY, aZ);}
	@Override public void onNeighborBlockChange(World aWorld, int aX, int aY, int aZ, Block aBlock) {super.onNeighborBlockChange(aWorld, aX, aY, aZ, aBlock); checkAndDropBlock(aWorld, aX, aY, aZ);}
	@Override public void updateTick(World aWorld, int aX, int aY, int aZ, Random aRandom) {checkAndDropBlock(aWorld, aX, aY, aZ);}
	@Override public boolean isOpaqueCube() {return F;}
	@Override public boolean renderAsNormalBlock() {return F;}
	@Override public boolean isNormalCube(IBlockAccess aWorld, int aX, int aY, int aZ)  {return F;}
	@Override public boolean isSideSolid(IBlockAccess aWorld, int aX, int aY, int aZ, ForgeDirection aDirection) {return F;}
	@Override public int getLightOpacity() {return LIGHT_OPACITY_NONE;}
	@Override public EnumPlantType getPlantType(IBlockAccess aWorld, int aX, int aY, int aZ) {return Water;}
	@Override public Block getPlant(IBlockAccess aWorld, int aX, int aY, int aZ) {return this;}
	@Override public int getPlantMetadata(IBlockAccess aWorld, int aX, int aY, int aZ) {return aWorld.getBlockMetadata(aX, aY, aZ);}
	@Override public float getBlockHardness(World aWorld, int aX, int aY, int aZ) {return Blocks.waterlily.getBlockHardness(aWorld, aX, aY, aZ);}
	@Override public float getExplosionResistance(int aMeta) {return Blocks.waterlily.getExplosionResistance(null);}
	@Override public int getItemStackLimit(ItemStack aStack) {return 64;}
	@Override public void getSubBlocks(Item aItem, CreativeTabs aTab, List aList) {for (int i = 0; i < 16; i++) aList.add(ST.make(aItem, 1, i));}
	
	public void checkAndDropBlock(World aWorld, int aX, int aY, int aZ) {
		if (!canBlockStay(aWorld, aX, aY, aZ)) {
			dropBlockAsItem(aWorld, aX, aY, aZ, aWorld.getBlockMetadata(aX, aY, aZ), 0);
			aWorld.setBlock(aX, aY, aZ, getBlockById(0), 0, 2);
		}
	}
	
	@Override
	public ItemStack onItemRightClick(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {
		MovingObjectPosition tPos = UT.Worlds.getMovingObjectPositionFromPlayer(aWorld, aPlayer, T);
		if (tPos == null || tPos.typeOfHit != MovingObjectPosition.MovingObjectType.BLOCK) return aStack;
		int aX = tPos.blockX, aY = tPos.blockY, aZ = tPos.blockZ;
		if (!aWorld.canMineBlock(aPlayer, aX, aY, aZ) || !aPlayer.canPlayerEdit(aX, aY, aZ, tPos.sideHit, aStack)) return aStack;
		if (aWorld.getBlock(aX, aY, aZ).getMaterial() == Material.water && aWorld.getBlockMetadata(aX, aY, aZ) == 0 && aWorld.isAirBlock(aX, aY+1, aZ)) {
			aWorld.setBlock(aX, aY+1, aZ, this, ST.meta(aStack), 3);
			if (!UT.Entities.hasInfiniteItems(aPlayer)) {aStack.stackSize--;}
		}
		return aStack;
	}
	
	@Override public AxisAlignedBB getCollisionBoundingBoxFromPool(World aWorld, int aX, int aY, int aZ) {return AxisAlignedBB.getBoundingBox(aX, aY, aZ, aX+1, aY+0.015625F, aZ+1);}
	@Override public AxisAlignedBB getSelectedBoundingBoxFromPool (World aWorld, int aX, int aY, int aZ) {return AxisAlignedBB.getBoundingBox(aX, aY, aZ, aX+1, aY+0.015625F, aZ+1);}
	@Override public void setBlockBoundsBasedOnState(IBlockAccess aWorld, int aX, int aY, int aZ) {setBlockBounds(0, 0, 0, 1, 0.015625F, 1);}
	
	@Override public IIcon getIcon(int aSide, int aMeta) {return mIcons[aMeta % mIcons.length].getIcon(0);}
	@Override public int getRenderType() {return RendererBlockTextured.INSTANCE==null?23:RendererBlockTextured.INSTANCE.mRenderID;}
	@Override public ITexture getTexture(int aRenderPass, byte aSide, ItemStack aStack) {return SIDES_VERTICAL[aSide] ? BlockTextureDefault.get(mIcons[ST.meta(aStack) % mIcons.length]) : null;}
	@Override public ITexture getTexture(int aRenderPass, byte aSide, boolean[] aShouldSideBeRendered, IBlockAccess aWorld, int aX, int aY, int aZ) {return SIDES_VERTICAL[aSide] ? BlockTextureDefault.get(mIcons[aWorld.getBlockMetadata(aX, aY, aZ) % mIcons.length]) : null;}
	@Override public boolean setBlockBounds(int aRenderPass, ItemStack aStack) {setBlockBounds(0, 0, 0, 1, 0.015625F, 1); return T;}
	@Override public boolean setBlockBounds(int aRenderPass, IBlockAccess aWorld, int aX, int aY, int aZ, boolean[] aShouldSideBeRendered) {setBlockBounds(0, 0, 0, 1, 0.015625F, 1); return T;}
	@Override public int getRenderPasses(ItemStack aStack) {return 1;}
	@Override public int getRenderPasses(IBlockAccess aWorld, int aX, int aY, int aZ, boolean[] aShouldSideBeRendered) {return 1;}
	@Override public IRenderedBlockObject passRenderingToObject(IBlockAccess aWorld, int aX, int aY, int aZ) {return null;}
	@Override public IRenderedBlockObject passRenderingToObject(ItemStack aStack) {return null;}
	@Override public boolean usesRenderPass(int aRenderPass, ItemStack aStack) {return T;}
	@Override public boolean usesRenderPass(int aRenderPass, IBlockAccess aWorld, int aX, int aY, int aZ, boolean[] aShouldSideBeRendered) {return T;}
}