package gregapi.worldgen;


import gregapi.data.CS.ConfigsGT;

import java.util.*;

import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraft.world.chunk.IChunkProvider;

/**
 * @author Gregorius Techneticies
 */
public abstract class WorldgenObject {
	
	public final String mWorldGenName;
	public final boolean mEnabled;
	private final Map<String, Boolean> mDimensionMap = new HashMap<String, Boolean>();
	
	public WorldgenObject(String aName, List aList, boolean aDefault) {
		mWorldGenName = aName==null?"":aName;
		mEnabled = ConfigsGT.WORLDGEN.get("worldgen", mWorldGenName, aDefault);
		if (mEnabled) aList.add(this);
	}
	
	/**
	 * @param aWorld The World Object
	 * @param aRandom The Random Generator to use
	 * @param aBiome The Name of the Biome (always != null)
	 * @param aDimensionType The Type of Worldgeneration to add. -1 = Nether, 0 = Overworld, +1 = End
	 * @param aChunkX xCoord of the Chunk
	 * @param aChunkZ zCoord of the Chunk
	 * @return if the Worldgeneration has been successfully completed
	 */
	public boolean executeWorldgen(World aWorld, Random aRandom, String aBiome, int aDimensionType, int aChunkX, int aChunkZ, IChunkProvider aChunkGenerator, IChunkProvider aChunkProvider) {
		return false;
	}
	
	/**
	 * @param aWorld The World Object
	 * @param aRandom The Random Generator to use
	 * @param aBiome The Name of the Biome (always != null)
	 * @param aDimensionType The Type of Worldgeneration to add. -1 = Nether, 0 = Overworld, +1 = End
	 * @param aChunkX xCoord of the Chunk
	 * @param aChunkZ zCoord of the Chunk
	 * @return if the Worldgeneration has been successfully completed
	 */
	public boolean executeWorldgen(World aWorld, Random aRandom, BiomeGenBase[][] aBiomes, Set<String> aBiomeNames, int aDimensionType, int aChunkX, int aChunkZ, IChunkProvider aChunkGenerator, IChunkProvider aChunkProvider) {
		return executeWorldgen(aWorld, aRandom, aBiomes[8][8].biomeName, aDimensionType, aChunkX, aChunkZ, aChunkGenerator, aChunkProvider);
	}
	
	/**
	 * @param aWorld The World Object
	 * @param aRandom The Random Generator to use
	 * @param aBiome The Name of the Biome (always != null)
	 * @param aDimensionType The Type of Worldgeneration to add. -1 = Nether, 0 = Overworld, +1 = End
	 * @param aChunkX xCoord of the Chunk
	 * @param aChunkZ zCoord of the Chunk
	 * @return if the Worldgeneration has been successfully completed
	 */
	public boolean executeCavegen(World aWorld, Random aRandom, String aBiome, int aDimensionType, int aChunkX, int aChunkZ, IChunkProvider aChunkGenerator, IChunkProvider aChunkProvider) {
		return false;
	}
	
	public boolean isGenerationAllowed(String aDimName, int aDimensionType, int aAllowedDimensionType) {
		aDimName = aDimName.replaceAll(" ", "_");
		Boolean tAllowed = mDimensionMap.get(aDimName);
		if (tAllowed == null) {
			boolean tValue = ConfigsGT.WORLDGEN.get("worldgen.dimensions."+mWorldGenName, aDimName.replaceAll(" ", "_"), aDimensionType == aAllowedDimensionType);
			mDimensionMap.put(aDimName, tValue);
			return tValue;
		}
		return tAllowed;
	}
	
	public boolean isGenerationAllowed(World aWorld, int aDimensionType, int aAllowedDimensionType) {
		return isGenerationAllowed(aWorld.provider.getDimensionName(), aDimensionType, aAllowedDimensionType);
	}
}